<?php
namespace ElementorTMcaps;

class TMcapsLoadElementor {

	private static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	private function include_widgets_files() {
		require_once( __DIR__ . '/widgets/meme-generator.php' );
	}

	public function loader_styles() {
		wp_enqueue_style( 'tmcaps-loader', plugins_url( '/css/loader.min.css', __FILE__ ), false, '1.0' );
	}

	public function widget_styles() {
		wp_register_style( 'tooltipster', plugins_url( '/css/tooltipster.min.css', __FILE__ ), false, '1.0' );
        wp_register_style( 'tmcaps', plugins_url( '/css/style.css', __FILE__ ), false, '1.0' );
	}

	public function register_widgets() {
		$this->include_widgets_files();

		\Elementor\Plugin::instance()->widgets_manager->register( new Widgets\TMcaps_Meme_Generator() );
	}

    public function widget_scripts() {
		wp_deregister_script('imagesloaded');
		wp_register_script( 'imagesloaded', plugins_url( '/js/imagesloaded.min.js', __FILE__ ), [ 'jquery' ], '4.1.4', true );
		wp_register_script( 'tooltipster', plugins_url( '/js/tooltipster.min.js', __FILE__ ), [ 'jquery' ], false, true );
		wp_register_script( 'fabric', plugins_url( '/js/fabric.min.js', __FILE__ ), [ 'jquery' ], false, true );
		wp_register_script( 'vanilla-picker', plugins_url( '/js/vanilla-picker.min.js', __FILE__ ), [ 'jquery' ], false, true );
        wp_register_script( 'tmcaps', plugins_url( '/js/tmcaps.min.js', __FILE__ ), [ 'jquery' ], false, true );
		wp_register_script( 'tmcaps-custom', plugins_url( '/js/custom.min.js', __FILE__ ), [ 'jquery' ], false, true );
	}

	public function widget_category($elements_manager) {
		$elements_manager->add_category(
			'tmcaps-widgets',
			[
				'title' => esc_html__( 'TM Caps', 'tmcaps' ),
				'icon' => 'fa fa-plug',
			]
		);
	}

	public function __construct() {
		add_action( 'wp_enqueue_scripts', [ $this, 'loader_styles' ], 1 );
		add_action( 'elementor/frontend/after_enqueue_styles', [ $this, 'widget_styles' ] );
        add_action( 'elementor/frontend/after_register_scripts', [ $this, 'widget_scripts' ] );
		add_action( 'elementor/elements/categories_registered', [ $this, 'widget_category' ] );
		add_action( 'elementor/widgets/register', [ $this, 'register_widgets' ] );
	}
}

TMcapsLoadElementor::instance();